@@ -6,18 +6,18 @@ use std::string::String;
 use std::ops::{self, Range, RangeBounds, Bound};
 use std::fmt::{Debug, Write};
 use std::fs::File;
+use std::path::Path;
 use rand::prelude::*;
-use bit_vec::BitVec;
 
 use super::*;
 
 /// Returns distance to leaves (only black nodes).
-fn check_tree_recursive<T, V, Ix>(tree: &IntervalMap<T, V, Ix>, index: Ix, upper_interval: &mut Interval<T>,
+fn validate_tree_recursive<T, V, Ix>(tree: &IntervalMap<T, V, Ix>, index: Ix, upper_interval: &mut Interval<T>,
     visited: &mut BitVec) -> u32
 where T: PartialOrd + Copy,
       Ix: IndexType,
 {
-    assert!(!visited[index.get()], "The tree contains a cycle: node {} was visited twice", index);
+    assert!(!visited.get(index.get()), "The tree contains a cycle: node {} was visited twice", index);
     visited.set(index.get(), true);
 
     let node = &tree.nodes[index.get()];
@@ -26,18 +26,18 @@ where T: PartialOrd + Copy,
     let right = node.right;
 
     let left_depth = if left.defined() {
-        if node.is_red() {
-            assert!(tree.nodes[left.get()].is_black(), "Red node {} has a red child {}", index, left);
+        if tree.is_red(index) {
+            assert!(tree.is_black(left), "Red node {} has a red child {}", index, left);
         }
-        Some(check_tree_recursive(tree, left, &mut down_interval, visited))
+        Some(validate_tree_recursive(tree, left, &mut down_interval, visited))
     } else {
         None
     };
     let right_depth = if right.defined() {
-        if node.is_red() {
-            assert!(tree.nodes[right.get()].is_black(), "Red node {} has a red child {}", index, right);
+        if tree.is_red(index) {
+            assert!(tree.is_black(right), "Red node {} has a red child {}", index, right);
         }
-        Some(check_tree_recursive(tree, right, &mut down_interval, visited))
+        Some(validate_tree_recursive(tree, right, &mut down_interval, visited))
     } else {
         None
     };
@@ -49,14 +49,18 @@ where T: PartialOrd + Copy,
         _ => {},
     }
     let depth = left_depth.or(right_depth).unwrap_or(0);
-    if node.is_black() {
+    if tree.is_black(index) {
         depth + 1
     } else {
         depth
     }
 }
 
-fn check<T: PartialOrd + Copy, V, Ix: IndexType>(tree: &IntervalMap<T, V, Ix>) {
+fn validate<T: PartialOrd + Copy, V, Ix: IndexType>(tree: &IntervalMap<T, V, Ix>, size: usize) {
+    assert_eq!(size, tree.len(), "Tree sizes do not match");
+    assert_eq!(size > 0, tree.root.defined(), "Tree root != size");
+    assert_eq!(tree.len(), tree.colors.len(), "Number of nodes != number of colors");
+
     if !tree.root.defined() {
         assert!(tree.nodes.is_empty(), "Non empty nodes with an empty root");
         return;
@@ -72,12 +76,12 @@ fn check<T: PartialOrd + Copy, V, Ix: IndexType>(tree: &IntervalMap<T, V, Ix>) {
     let node = &tree.nodes[tree.root.get()];
     let mut interval = node.interval.clone();
     let mut visited = BitVec::from_elem(tree.nodes.len(), false);
-    check_tree_recursive(tree, tree.root, &mut interval, &mut visited);
+    validate_tree_recursive(tree, tree.root, &mut interval, &mut visited);
     assert!(interval == node.subtree_interval, "Interval != subtree interval for node {}", tree.root);
-}
 
-struct NaiveIntervalMap<T: PartialOrd + Copy, V> {
-    nodes: Vec<(Range<T>, V)>,
+    for i in 0..tree.len() {
+        assert!(visited.get(i), "The tree is disjoint: node {} has no connection to the root", i);
+    }
 }
 
 fn intersects<T: PartialOrd, R: RangeBounds<T>>(range: &Range<T>, query: &R) -> bool {
@@ -93,6 +97,14 @@ fn intersects<T: PartialOrd, R: RangeBounds<T>>(range: &Range<T>, query: &R) ->
     })
 }
 
+fn range_eq<T: PartialOrd>(a: &Range<T>, b: &Range<T>) -> bool {
+    a.start_bound() == b.start_bound() && a.end_bound() == b.end_bound()
+}
+
+struct NaiveIntervalMap<T: PartialOrd + Copy, V> {
+    nodes: Vec<(Range<T>, V)>,
+}
+
 impl<T: PartialOrd + Copy, V> NaiveIntervalMap<T, V> {
     fn new() -> Self {
         Self {
@@ -100,6 +112,10 @@ impl<T: PartialOrd + Copy, V> NaiveIntervalMap<T, V> {
         }
     }
 
+    fn len(&self) -> usize {
+        self.nodes.len()
+    }
+
     fn insert(&mut self, range: Range<T>, value: V) {
         self.nodes.push((range, value));
     }
@@ -108,6 +124,14 @@ impl<T: PartialOrd + Copy, V> NaiveIntervalMap<T, V> {
         self.nodes.iter().filter(move |(range, _value)| intersects(range, &query))
             .map(|(range, value)| (range.clone(), value))
     }
+
+    fn all_matching<'a>(&'a self, query: Range<T>) -> impl Iterator<Item = &V> + 'a {
+        self.nodes.iter().filter(move |(range, _value)| range_eq(&range, &query)).map(|(_range, value)| value)
+    }
+
+    fn remove_random(&mut self, rng: &mut impl Rng) -> (Range<T>, V) {
+        self.nodes.swap_remove(rng.gen_range(0..self.nodes.len()))
+    }
 }
 
 fn generate_ordered_pair<T: PartialOrd + Copy, F: FnMut() -> T>(generator: &mut F, forbid_eq: bool) -> (T, T) {
@@ -126,15 +150,18 @@ fn generate_ordered_pair<T: PartialOrd + Copy, F: FnMut() -> T>(generator: &mut
 fn modify_maps<T, F>(naive: &mut NaiveIntervalMap<T, u32>, tree: &mut IntervalMap<T, u32>, n_inserts: u32,
         mut generator: F) -> String
 where T: PartialOrd + Copy + Debug,
-      F: FnMut() -> T,
+      F: FnMut() -> Range<T>,
 {
     let mut history = String::new();
     for i in 0..n_inserts {
-        let (a, b) = generate_ordered_pair(&mut generator, true);
-        let range = a..b;
+        let range = generator();
         writeln!(history, "insert({:?})", range).unwrap();
         naive.insert(range.clone(), i);
-        tree.insert(range.clone(), i);
+        if let Some(value) = tree.insert(range.clone(), i) {
+            let i = naive.nodes.iter().position(|(range2, _value2)| range == *range2).unwrap();
+            assert_eq!(naive.nodes[i].1, value);
+            naive.nodes.swap_remove(i);
+        }
     }
     history
 }
@@ -149,18 +176,24 @@ where T: PartialOrd + Copy + Debug,
     res
 }
 
-fn generate_int(range: Range<i32>) -> impl (FnMut() -> i32) {
+fn generate_int(a: i32, b: i32) -> impl (FnMut() -> i32) {
     let mut rng = thread_rng();
-    move || rng.gen_range(range.start..range.end)
+    move || rng.gen_range(a..b)
 }
 
-fn generate_float(range: Range<f64>) -> impl (FnMut() -> f64) {
+fn generate_float(range: f64) -> impl (FnMut() -> f64) {
     let mut rng = thread_rng();
-    move || rng.gen_range(range.start..range.end)
+    move || rng.gen::<f64>() * range
+}
+
+fn generate_float_rounding() -> impl (FnMut() -> f64) {
+    const MULT: f64 = 1e8;
+    let mut rng = thread_rng();
+    move || (rng.gen::<f64>() * MULT).round() / MULT
 }
 
 fn generate_range<T: PartialOrd + Copy + Debug, F: FnMut() -> T>(mut generator: F)
-        -> impl (FnMut() -> ops::Range<T>) {
+        -> impl (FnMut() -> Range<T>) {
     move || {
         let (a, b) = generate_ordered_pair(&mut generator, true);
         a..b
@@ -238,6 +271,39 @@ where T: PartialOrd + Copy + Debug
     }
 }
 
+fn compare_match_results<T>(naive: &NaiveIntervalMap<T, u32>, tree: &IntervalMap<T, u32>, history: &str, range: Range<T>)
+where T: PartialOrd + Copy + Clone + Debug
+{
+    let values: Vec<u32> = naive.all_matching(range.clone()).map(|v| *v).collect();
+    let mut correct = true;
+    correct &= values.is_empty() != tree.contains(range.clone());
+    correct &= match tree.get(range.clone()) {
+        Some(value) => values.contains(&value),
+        None => values.is_empty(),
+    };
+    if !correct {
+        println!("Range: {:?},   values: {:?},   tree.get: {:?}", range, values, tree.get(range.clone()));
+        println!("{}", history);
+        println!();
+        panic!();
+    }
+}
+
+fn compare_exact_matching<T, F>(naive: &NaiveIntervalMap<T, u32>, tree: &IntervalMap<T, u32>, history: &str,
+    mut generator: F)
+where T: PartialOrd + Copy + Debug,
+      F: FnMut() -> Range<T>,
+{
+    for (range, _value) in &naive.nodes {
+        compare_match_results(naive, tree, history, range.clone());
+    }
+
+    for _ in 0..naive.nodes.len() {
+        compare_match_results(naive, tree, history, generator());
+    }
+}
+
+#[cfg(feature = "serde")]
 fn compare_iterators<T, I, J>(mut iter1: I, mut iter2: J, history: &str)
 where
     T: PartialEq + Debug,
@@ -261,22 +327,24 @@ fn test_int_inserts() {
     const COUNT: u32 = 1000;
     let mut naive = NaiveIntervalMap::new();
     let mut tree = IntervalMap::new();
-    let history = modify_maps(&mut naive, &mut tree, COUNT, generate_int(0..100));
+    let mut generator = generate_int(0, 100);
+    let history = modify_maps(&mut naive, &mut tree, COUNT, generate_range(&mut generator));
 
-    let output_path = std::path::Path::new("tests/data/int.dot");
+    let output_path = Path::new("tests/data/int.dot");
     let folders = output_path.parent().unwrap();
     std::fs::create_dir_all(folders).unwrap();
     let f = File::create(output_path).unwrap();
     tree.write_dot(f).unwrap();
-    check(&tree);
+    validate(&tree, naive.len());
     compare_extremums(&naive, &tree, &history);
+    compare_exact_matching(&naive, &tree, &history, generate_range(&mut generator));
 
-    search_rand(&mut naive, &mut tree, COUNT, generate_range(generate_int(0..100)), &history);
-    search_rand(&mut naive, &mut tree, COUNT, generate_range_from(generate_int(0..100)), &history);
+    search_rand(&mut naive, &mut tree, COUNT, generate_range(&mut generator), &history);
+    search_rand(&mut naive, &mut tree, COUNT, generate_range_from(&mut generator), &history);
     search_rand(&mut naive, &mut tree, 1, generate_range_full, &history);
-    search_rand(&mut naive, &mut tree, COUNT, generate_range_incl(generate_int(0..100)), &history);
-    search_rand(&mut naive, &mut tree, COUNT, generate_range_to(generate_int(0..100)), &history);
-    search_rand(&mut naive, &mut tree, COUNT, generate_range_to_incl(generate_int(0..100)), &history);
+    search_rand(&mut naive, &mut tree, COUNT, generate_range_incl(&mut generator), &history);
+    search_rand(&mut naive, &mut tree, COUNT, generate_range_to(&mut generator), &history);
+    search_rand(&mut naive, &mut tree, COUNT, generate_range_to_incl(&mut generator), &history);
 }
 
 #[test]
@@ -284,22 +352,42 @@ fn test_float_inserts() {
     const COUNT: u32 = 1000;
     let mut naive = NaiveIntervalMap::new();
     let mut tree = IntervalMap::new();
-    let history = modify_maps(&mut naive, &mut tree, COUNT, generate_float(0.0..1.0));
+    let mut generator = generate_float(1000.0);
+    let history = modify_maps(&mut naive, &mut tree, COUNT, generate_range(&mut generator));
 
-    let output_path = std::path::Path::new("tests/data/float.dot");
+    let output_path = Path::new("tests/data/float.dot");
     let folders = output_path.parent().unwrap();
     std::fs::create_dir_all(folders).unwrap();
     let f = File::create(output_path).unwrap();
     tree.write_dot(f).unwrap();
-    check(&tree);
+    validate(&tree, naive.len());
     compare_extremums(&naive, &tree, &history);
 
-    search_rand(&mut naive, &mut tree, COUNT, generate_range(generate_float(0.0..1.0)), &history);
-    search_rand(&mut naive, &mut tree, COUNT, generate_range_from(generate_float(0.0..1.0)), &history);
+    search_rand(&mut naive, &mut tree, COUNT, generate_range(&mut generator), &history);
+    search_rand(&mut naive, &mut tree, COUNT, generate_range_from(&mut generator), &history);
     search_rand(&mut naive, &mut tree, 1, generate_range_full, &history);
-    search_rand(&mut naive, &mut tree, COUNT, generate_range_incl(generate_float(0.0..1.0)), &history);
-    search_rand(&mut naive, &mut tree, COUNT, generate_range_to(generate_float(0.0..1.0)), &history);
-    search_rand(&mut naive, &mut tree, COUNT, generate_range_to_incl(generate_float(0.0..1.0)), &history);
+    search_rand(&mut naive, &mut tree, COUNT, generate_range_incl(&mut generator), &history);
+    search_rand(&mut naive, &mut tree, COUNT, generate_range_to(&mut generator), &history);
+    search_rand(&mut naive, &mut tree, COUNT, generate_range_to_incl(&mut generator), &history);
+}
+
+#[test]
+fn test_from_sorted() {
+    const COUNT: u32 = 1000;
+    let mut vec = Vec::new();
+    let mut map: IntervalMap<_, _, u32> = IntervalMap::from_sorted(vec.clone().into_iter());
+    validate(&map, 0);
+
+    for i in 0..COUNT {
+        vec.push((i..i+1, i));
+        map = IntervalMap::from_sorted(vec.clone().into_iter());
+        assert_eq!(map.len(), vec.len());
+        validate(&map, vec.len());
+    }
+
+    for (range, value) in vec {
+        assert_eq!(map.get(range), Some(&value));
+    }
 }
 
 #[cfg(feature = "serde")]
@@ -308,9 +396,9 @@ fn test_serde() {
     const COUNT: u32 = 1000;
     let mut naive = NaiveIntervalMap::new();
     let mut tree: IntervalMap<i32, u32> = IntervalMap::new();
-    let history = modify_maps(&mut naive, &mut tree, COUNT, generate_int(0..10000));
+    let history = modify_maps(&mut naive, &mut tree, COUNT, generate_range(generate_int(0, 10000)));
 
-    let json_path = std::path::Path::new("tests/data/serde.json");
+    let json_path = Path::new("tests/data/serde.json");
     let folders = json_path.parent().unwrap();
     std::fs::create_dir_all(folders).unwrap();
     let output = File::create(json_path).unwrap();
@@ -320,3 +408,40 @@ fn test_serde() {
     let tree2: IntervalMap<i32, u32> = serde_json::from_reader(input).unwrap();
     compare_iterators(tree.iter(..), tree2.iter(..), &history);
 }
+
+fn removal_with_insert_chance(insert_chance: f64, count: u32) {
+    let mut range_generator = generate_range(generate_float_rounding());
+    let mut naive = NaiveIntervalMap::<f64, u32>::new();
+    let mut tree = IntervalMap::<f64, u32>::new();
+
+    let mut rng = thread_rng();
+    for i in 0..count {
+        let r = rng.gen::<f64>();
+        if naive.len() == 0 || r <= insert_chance {
+            let range = range_generator();
+            println!("map.insert({:?}, {});", range, i);
+            naive.insert(range.clone(), i);
+            tree.insert(range, i);
+        } else {
+            let (range, value) = naive.remove_random(&mut rng);
+            println!("map.remove({:?}); // -> {}", range, value);
+            let rm_val = tree.remove(range.clone());
+            assert_eq!(rm_val, Some(value));
+            validate(&tree, naive.len());
+        }
+    }
+    println!("-------------")
+}
+
+#[test]
+fn test_removal() {
+    for _ in 0..10 {
+        removal_with_insert_chance(0.4, 10000);
+    }
+    for _ in 0..10 {
+        removal_with_insert_chance(0.6, 10000);
+    }
+    for _ in 0..10 {
+        removal_with_insert_chance(0.8, 10000);
+    }
+}
